@@ -1,4 +1,4 @@
-# SPDX-FileCopyrightText: 2021 Computer Assisted Medical Interventions Group, DKFZ
+# SPDX-FileCopyrightText: 2021 Division of Intelligent Medical Systems, DKFZ
 # SPDX-FileCopyrightText: 2021 Janek Groehl
 # SPDX-License-Identifier: MIT
 
@@ -751,6 +751,32 @@ class Tags:
     Usage: adapter PyTorchDASAdapter
     """
 
+    RECONSTRUCTION_PERFORM_RESAMPLING_FOR_FFT = ("reconstruction_perform_resampling_for_fft",
+                                    (bool, np.bool, np.bool_))
+    """
+    Whether the data is resampled to a power of 2 in time dimension before applying the FFT 
+    and resampled back after filtering for performance reasons. Default should be False\n
+    Usage: adapter reconstruction_utils
+    """
+
+    BANDPASS_FILTER_METHOD = ("bandpass_filtering_method", str)
+    """
+    Choice of the bandpass filtering method used, i.e. tukey or butterworth filter .\n
+    Usage: ReconstructionAdapterBase
+    """
+
+    TUKEY_BANDPASS_FILTER = "tukey_bandpass_filter"
+    """
+    Corresponds to the tukey bandpass filter\n
+    Usage: reconstruction utils
+    """
+
+    BUTTERWORTH_BANDPASS_FILTER = "butterworth_bandpass_filter"
+    """
+    Corresponds to the tukey bandpass filter\n
+    Usage: reconstruction utils
+    """
+
     TUKEY_WINDOW_ALPHA = ("tukey_window_alpha", (int, np.integer, float))
     """
     Sets alpha value of Tukey window between 0 (similar to box window) and 1 (similar to Hann window).
@@ -758,6 +784,13 @@ class Tags:
     Usage: adapter PyTorchDASAdapter
     """
 
+    BUTTERWORTH_FILTER_ORDER = ("butterworth_filter_order", (int, np.integer))
+    """
+    Sets the order of the filter, usually between 1 and 5.
+    Default is 1\n
+    Usage: reconstruction utils
+    """
+
     BANDPASS_CUTOFF_LOWPASS = ("bandpass_cuttoff_lowpass", (int, np.integer, float))
     """
     Sets the cutoff threshold in Hz for lowpass filtering, i.e. upper limit of the tukey filter. Default is 8 MHz\n
